/*
 * Copyright 2008 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _CONDOR_ALLOW_OPEN
#include "condor_common.h"

#include "../condor_master.V6/MasterPlugin.h"
#include "../condor_master.V6/master.h"

#include "condor_classad.h"

#include "../condor_daemon_core.V6/condor_daemon_core.h"

#include "condor_config.h"

#include "qpid/management/Manageable.h"
#include "qpid/management/ManagementObject.h"
#include "qpid/agent/ManagementAgent.h"

#include "Master.h"

#include "ArgsMasterStop.h"
#include "ArgsMasterStart.h"

#include "PoolUtils.h"

#include "MgmtConversionMacros.h"

extern DaemonCore *daemonCore;

extern class Daemons daemons;

using namespace std;
using namespace qpid::management;
using namespace qmf::mrg::grid;


struct MasterObject : public Manageable
{
	Master *mgmtObject;

	MasterObject(ManagementAgent *agent)
	{
		mgmtObject = new Master(agent, this);

		agent->addObject(mgmtObject, 1);
	}

	~MasterObject() { };

	ManagementObject *GetManagementObject(void) const
	{
		return mgmtObject;
	}

	status_t ManagementMethod(uint32_t methodId, Args &args, string &text)
	{
		switch (methodId) {
		case Master::METHOD_STOP:
			return Stop(((ArgsMasterStop &) args).i_Subsystem, text);
		case Master::METHOD_START:
			return Start(((ArgsMasterStart &) args).i_Subsystem, text);
		}

		return STATUS_NOT_IMPLEMENTED;
	}

	status_t Stop(string subsys, string &text)
	{
		class daemon *daemon;

		dprintf(D_ALWAYS, "Received Stop(%s)\n", subsys.c_str());

		if (!(daemon = daemons.FindDaemon(subsys.c_str()))) {
			text = "Unknown subsystem: " + subsys;

			dprintf(D_ALWAYS, "ERROR: %s\n", text.c_str());

			return STATUS_USER + 1;
		}

		daemon->Hold(true);
		daemon->Stop();

		return STATUS_OK;
	}

	status_t Start(string subsys, string &text)
	{
		class daemon *daemon;

		dprintf(D_ALWAYS, "Received Start(%s)\n", subsys.c_str());

		if (!(daemon = daemons.FindDaemon(subsys.c_str()))) {
			text = "Unknown subsystem: " + subsys;

			dprintf(D_ALWAYS, "ERROR: %s\n", text.c_str());

			return STATUS_USER + 1;
		}

			// XXX: daemon::on_hold may be useful to avoid changing
			// state on daemon::Start() failure

		daemon->Hold(false);

		if (!daemon->Start()) {
			text = "Unable to start subsystem: " + subsys;

			dprintf(D_ALWAYS, "ERROR: %s\n", text.c_str());

			return STATUS_USER + 2;
		}

		return STATUS_OK;
	}

	void update(const ClassAd &ad)
	{
		MGMT_DECLARATIONS;

		mgmtObject->set_Pool(GetPoolName());

		STRING(CondorPlatform);
		STRING(CondorVersion);
		TIME_INTEGER(DaemonStartTime);
		STRING(Machine);
		STRING(MyAddress);
		STRING(Name);
		STRING(PublicNetworkIpAddr);
		INTEGER(RealUid);

			//INTEGER(WindowsBuildNumber);
			//INTEGER(WindowsMajorVersion);
			//INTEGER(WindowsMinorVersion);

		INTEGER(MonitorSelfAge);
		DOUBLE(MonitorSelfCPUUsage);
		DOUBLE(MonitorSelfImageSize);
		INTEGER(MonitorSelfRegisteredSocketCount);
		INTEGER(MonitorSelfResidentSetSize);

 			// XXX: While we do not have the System's UUID we will use
			// its name. Right now this is a placeholder so when we do
			// get a UUID users of the System attribute don't have
			// to change.
		mgmtObject->set_System(mgmtObject->get_Machine());
	}
};


struct MgmtMasterPlugin : public Service, MasterPlugin
{
		// ManagementAgent::Singleton cleans up the ManagementAgent
		// instance if there are no ManagementAgent::Singleton's in
		// scope!
	ManagementAgent::Singleton *singleton;

	MasterObject *master;

	void
	initialize()
	{
		char *host;
		int port;
		char *tmp;
		string storefile;

		dprintf(D_FULLDEBUG, "MgmtMasterPlugin: Initializing...\n");

		singleton = new ManagementAgent::Singleton();

		ManagementAgent *agent = singleton->getInstance();

		Master::registerSelf(agent);

		port = param_integer("QMF_BROKER_PORT", 5672);
		if (NULL == (host = param("QMF_BROKER_HOST"))) {
			host = strdup("localhost");
		}

		tmp = param("QMF_STOREFILE");
		if (NULL == tmp) {
			storefile = ".master_storefile";
		} else {
			storefile = tmp;
			free(tmp); tmp = NULL;
		}

		agent->init(string(host), port,
					param_integer("QMF_UPDATE_INTERVAL", 10),
					true,
					storefile);

		free(host);

		master = new MasterObject(agent);

		ReliSock *sock = new ReliSock;
		if (!sock) {
			EXCEPT("Failed to allocate Mgmt socket");
		}
		if (!sock->assign(agent->getSignalFd())) {
			EXCEPT("Failed to bind Mgmt socket");
		}
		int index;
		if (-1 == (index =
				   daemonCore->Register_Socket((Stream *) sock,
											   "Mgmt Method Socket",
											   (SocketHandlercpp)
											   &MgmtMasterPlugin::HandleMgmtSocket,
											   "Handler for Mgmt Methods.",
											   this))) {
			EXCEPT("Failed to register Mgmt socket");
		}
	}

	void
	shutdown()
	{
		if (!param_boolean("QMF_DELETE_ON_SHUTDOWN", false)) {
			return;
		}

		dprintf(D_FULLDEBUG, "MgmtMasterPlugin: shutting down...\n");
		if (singleton) {
			dprintf(D_FULLDEBUG, "...deleting ManagementAgent\n");
			delete singleton;
			singleton = NULL;
			dprintf(D_FULLDEBUG, "...deleted ManagementAgent\n");
		}
	}

	void
	update(const ClassAd *ad)
	{
		master->update(*ad);
	}

	int
	HandleMgmtSocket(Service *, Stream *)
	{
		singleton->getInstance()->pollCallbacks();

		return KEEP_STREAM;
	}
};

static MgmtMasterPlugin instance;
